#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class Collections(object):
    """Simple class to parse collections json from INPE STAC"""

    def __init__(self, data):
        """
        docstring
        """
        self._data = data

    def __getitem__(self, k):
        """
        docstring
        """
        try:
            return next(filter(lambda i: i['id'] == k, self._data['collections']))
        except StopIteration:
            raise KeyError(k)

    def __iter__(self):
        """
        docstring
        """
        for i in [(collection['id'], collection['description']) for collection in self._data['collections']]:
            yield i

    def get_spatial_extent(self, id):
        """
        docstring
        """
        return self.__getitem__(id)['extent']['spatial']

    def get_temporal_extent(self, id):
        """
        docstring
        """
        return self.__getitem__(id)['extent']['temporal']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .item import Item


class ItemCollection(object):
    """Class to parse json from INPE STAC Catalog"""

    def __init__(self, data):
        self._data = data

    @property
    def returned(self):
        return self._data['context']['returned']

    @property
    def matched(self):
        return self._data['context']['matched']

    @property
    def limit(self):
        return self._data['context']['limit']

    @property
    def complete(self):
        return self._data['context']['returned'] == self._data['context']['matched']

    @property
    def featurescollection(self):  # FeaturesCollection GeoJSON ? ['type', 'id', 'geometry', 'bbox', 'properties']
        return {'type': 'FeatureCollection', 'features': list(self.features('type', 'id', 'collection', 'geometry', 'bbox', 'properties', 'assets'))}

    def items(self):
        for feat in self._data['features']:
            yield Item(feat)

    def features(self, *keys):
        keys = list(filter(lambda k: k in ['type', 'id', 'collection', 'geometry', 'bbox', 'properties', 'assets', 'links'], keys))
        if keys:
            for feat in self._data['features']:
                yield {key: feat[key] for key in keys}
        else:
            for feat in self._data['features']:
                yield feat

    def __len__(self):
        return self._data['context']['returned']

    def __getitem__(self, k):
        """
        docstring
        """
        try:
            return Item(next(filter(lambda i: i['id'] == k, self._data['features'])))
        except StopIteration:
            raise KeyError(k)

    def __contains__(self, k):
        """
        docstring
        """
        if isinstance(k, Item):
            return any(i['id'] == k.id for i in self._data['features'])
        else:
            return any(i['id'] == k for i in self._data['features'])

    def __iter__(self):
        """
        docstring
        """
        return self.items()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
        name='cbers4asat',
        version='0.5',
        description='Biblioteca Python para consultar o catálogo e realizar operações com dados do CBERS4A',
        url='https://github.com/gabriel-russo/cbers4asat',
        author='Gabriel Russo',
        author_email='gabrielrusso@protonmail.com',
        license='MIT',
        license_files='LICENSE',
        long_description=long_description,
        long_description_content_type="text/markdown",
        classifiers=[
                "Programming Language :: Python :: 3.7",
                "Programming Language :: Python :: 3.8",
                "Programming Language :: Python :: 3.9",
                "Programming Language :: Python :: 3.10",
                "License :: OSI Approved :: MIT License",
                "Operating System :: OS Independent",
                "Topic :: Scientific/Engineering :: GIS",
                "Development Status :: 3 - Alpha"
        ],
        packages=find_packages(),
        install_requires=[
                "geopandas >= 0.9",
                "requests >= 2.25.1",
                "pandas >= 1.3.5"
        ]
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime
import requests
from os.path import join


class Item(object):
    """Class to parse items from INPE STAC Catalog"""

    def __init__(self, feature):
        self._feature = feature

    def __str__(self):
        """
        docstring
        """
        row = "{id:<25s} {date:^10s} {cloud:>15.1f}".format
        return row(id=self.id, date=self.date, cloud=self.get_property('cloud_cover'))

    def _repr_html_(self):
        """
        html representation for jupyter notebook
        """

        html = f"""
        <html>
        <body>
        <img style="display: block; margin-left: auto; margin-right: auto;" src="{self.thumbnail}" width="296" border="0" />
        </body>
        </html>
        """

        return html

    def repr_html(self, width=450):
        """
        html representation to generate a html catalog
        """
        html = f"""
        <div>
            <h2>{self.id}</h2>
            <span>Satelitte: {self.satellite}</span>
            <span>Sensor: {self.sensor}</span>
            <span>Collection: {self.collection}</span>
            <span>Cloud cover: {self.cloud_cover}</span>
            <span>Datetime: {self.get_datetime_fmt(fmt='%Y-%m-%d-%H:%m:%S')}</span>
            <span>bbox: {self.bbox}</span> 
            <img src="{self.thumbnail}" width="{width}"/>
        </div>
        """
        return html

    def __lt__(self, other):
        """
        docstring
        """
        return self.get_property('cloud_cover', 0) < other.get_property('cloud_cover', 0)

    @property
    def __geo_interface__(self):
        """
        Simple GeoJSON-like interface
        """
        return self._feature['geometry']

    @property
    def id(self):
        """
        Provider identifier globally unique.
        """
        return self._feature['id']

    @property
    def date(self):
        """
        The acquisition date of the asset.
        """
        return self.get_datetime_fmt()

    @property
    def bbox(self):
        """
        Bounding Box of the asset.
        """
        return self._feature.get('bbox')

    @property
    def collection(self):
        """
        The id of the STAC Collection this Item references to.
        """
        return self._feature.get('collection')

    @property
    def thumbnail(self):
        """
        Thumbnail image link.
        """
        return self._feature['assets']['thumbnail']['href']

    @property
    def assets(self):
        """
        Unique keys of asset objects that can be downloaded.
        """
        return list(self._feature['assets'].keys())

    @property
    def cloud_cover(self):
        """
        Cloud coverage %
        """
        return self.get_property('cloud_cover')

    @property
    def path_row(self):
        """
        Return a string 'PATH_ROW'
        """
        return str(self.get_property('path', '')) + '_' + str(self.get_property('row', ''))

    @property
    def sensor(self):
        """
        Return the sensor
        """
        return self.get_property('sensor')

    @property
    def satellite(self):
        """
        Return the satellite
        """
        return self.get_property('satellite')

    @property
    def html(self):
        """
        Html to show image preview
        """

        return self.repr_html()

    def get_datetime(self):
        """
        The acquisition date and time of the assets
        as an object of the class datetime.datetime.
        """
        try:
            return datetime.strptime(self.get_property('datetime'), '%Y-%m-%dT%H:%M:%S')
        except (ValueError, TypeError):
            return None

    def get_datetime_fmt(self, fmt='%Y-%m-%d'):
        """
        Return a string representing the acquisition date and time, controlled by a format string.
        """
        try:
            return self.get_datetime().strftime(fmt)
        except AttributeError:
            return None

    def get_property(self, p, v=None):
        """
        Get additional metadata fields.
        None if it does not exist.
        """
        return self._feature['properties'].get(p, v)

    def url(self, asset):
        """
        Get asset url
        """
        return self._feature['assets'][asset]['href']

    def download(self, asset, credential, outdir, session=requests.Session()):
        """
        Download the asset.
        """
        url = self.url(asset)
        filename = url.split("/")[-1]
        outfile = join(outdir, filename)
        r = session.get(url, params={'email': credential, 'item_id': self.id, 'collection': self.collection}, stream=True, allow_redirects=True)
        if r.status_code == 200:
            with open(outfile, 'wb') as f:
                for ch in r.iter_content(chunk_size=4096): # (page size 4Kb)
                    if ch:
                        f.write(ch)

            return outfile
        else:
            return 'ERROR in ' + url + ' (' + r.reason + ')'
